package views;

import javafx.beans.property.Property;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.ColorPicker;
import javafx.scene.control.Label;
import javafx.scene.control.Spinner;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import model.Manager;
import model.actualiseur.ActualiseurTourUnParUn;
import model.cellule.Cellule;
import model.plateau.Plateau;

import java.util.Random;

public class VueJeu {
    @FXML
    private Button random;

    @FXML
    private GridPane map;

    @FXML
    private ColorPicker color;

    @FXML
    private Spinner<Integer> rowGame;

    @FXML
    private Spinner<Integer> colGame;

    @FXML
    private Label nbRowGame;

    @FXML
    private Label nbColGame;

    @FXML
    private Label numTour;

    private Manager manager;
    private Color deathColor;
    private Plateau plateau;

    public void createGrid() {
        map.getChildren().clear();
        for(int i=0; i < colGame.getValue().intValue(); ++i) {
            for(int j=0; j < rowGame.getValue().intValue(); ++j) {
                Rectangle rect = new Rectangle(15, 15, color.getValue());
                Cellule c = manager.getActualiseurCellule().getArbitre().getPlateau().getCell(j, i);
                rect.fillProperty().bindBidirectional(c.activeColorProperty());
                rect.setOnMouseClicked((src)->manager.inverserEtatCellule(c));
                map.add(rect, i, j);
            }
        }
    }

    public void generateraRandom() {
        resetGrid();
        int largeur = manager.getActualiseurCellule().getArbitre().getPlateau().getLargeur();
        int longueur = manager.getActualiseurCellule().getArbitre().getPlateau().getLongueur();
        for(int i=0; i<(longueur+largeur)/2; ++i) {
            Random random = new Random();
            manager.getActualiseurCellule().getArbitre().getPlateau().getCell(random.nextInt(longueur), random.nextInt(largeur)).setAlive(true);
        }
    }

    public void initialize() {
        manager = new Manager();
        deathColor = Color.BLACK;
        rowGame.getValueFactory().valueProperty().bindBidirectional((Property) manager.getActualiseurCellule().getArbitre().getPlateau().largeurProperty());
        colGame.getValueFactory().valueProperty().bindBidirectional((Property) manager.getActualiseurCellule().getArbitre().getPlateau().longueurProperty());
        manager.getActualiseurCellule().getArbitre().getPlateau().longueurProperty().addListener((src)->resetGrid());
        manager.getActualiseurCellule().getArbitre().getPlateau().largeurProperty().addListener((src)->resetGrid());
        colGame.getValueFactory().setValue(10);
        rowGame.getValueFactory().setValue(10);

        color.valueProperty().bindBidirectional(Cellule.livingColorProperty());

        nbColGame.setText(colGame.getValue().toString());
        nbRowGame.setText(rowGame.getValue().toString());
        numTour.textProperty().bind(((ActualiseurTourUnParUn)manager.getActualiseurTour()).cptTourProperty().asString());
        nbColGame.textProperty().bind(manager.getActualiseurCellule().getArbitre().getPlateau().longueurProperty().asString());
        nbRowGame.textProperty().bind(manager.getActualiseurCellule().getArbitre().getPlateau().largeurProperty().asString());
    }

    public void startGame(ActionEvent actionEvent) {
        manager.lancerJeu();
    }

    public void pauseGame(ActionEvent actionEvent) {
        manager.pauseJeu();
    }

    public void resetGame(ActionEvent actionEvent){
        manager.stoperJeu();
        resetGrid();
    }

    public void resetGrid(){
        manager.getActualiseurCellule().getArbitre().getPlateau().resetGrille();
        createGrid();
    }
}
